# -*- coding: utf-8 -*-
"""
Created on Fri Sep 11 14:03:23 2021

@author: Eli Shamis
"""
import socketio 
from neurospeed.utils.api_config_service import ApiConfig


# target_is_customer_room 
class CustomerRoom_Handler:    
    
    def __init__(self, customer_auth_handler):
        self._auth_handler = customer_auth_handler
        self._hia_connection_external_event_handler = None
        self._hia_disconnect_external_event_handler = None
        self._customerRoom_api = None

    def connect(self):
         # Create Customer_TargetRoom_Api
        self._customerRoom_api = self.CustomerRoom_Api(self._auth_handler)
        
        # attach relevant internal handlers for socket.io events
        self._customerRoom_api.set_connection_handler(self.customerRoom_socket_connection_handler)
        self._customerRoom_api.set_disconnect_handler(self.customerRoom_socket_disconnect_handler)
        self._customerRoom_api.set_hia_events_handler(self.customerRoom_hia_events_handler)
    
        self._customerRoom_api.connect()
        
        
    def disconnect(self):
            self._customerRoom_api.disconnect()
        
        
    def customerRoom_socket_connection_handler(self):
        print('customer connected to customerRoom')
        pass;
    
    def customerRoom_socket_disconnect_handler(self):
        print('customer disconnected from customerRoom')
        pass;
       
    # see README(4) for payload structure  
    def customerRoom_hia_events_handler(self, payload):

        event_type = payload["type"]
        username = payload["username"]
        hia_id = payload["hia_id"]
        sensors = payload["sensors"]
        
        if (event_type == 'connect'): 
            self.hia_connection_event_handler(payload)
        
        if (event_type == 'disconnect'):
            self.hia_disconnect_event_handler(payload)    
            
    
    # inner hia_connection event handler
    def hia_connection_event_handler(self, event):
        # propogate event to main program (or to any other source, depends on the external_handler callback)
        if (self._hia_connection_external_event_handler != None):
            self._hia_connection_external_event_handler(event)
    
    # inner hia_disconnect event handler
    def hia_disconnect_event_handler(self, event):
        # propogate event to main program (or to any other source, depends on the external_handler callback)
        if (self._hia_disconnect_external_event_handler != None):
            self._hia_disconnect_external_event_handler(event)      
  
    
    # setter for external handler
    def set_hia_connection_external_handler(self, handler):
        self._hia_connection_external_event_handler = handler
    
    # setter for external handler
    def set_hia_disconnect_external_handler(self, handler):
        self._hia_disconnect_external_event_handler = handler
        
        
    # Customer_Room_Api api class for receiving events across all user devices
    class CustomerRoom_Api:    
        
        def __init__(self, config):
            api_config = ApiConfig()
            self._config = config
            self._socket_url =  api_config.get_socket_url()
            self._access_token = config.get_access_token()
      
            logger_on = True
            if self._config.is_verbose_log() == False:
                logger_on = False
                
            self._sio = socketio.Client(logger=logger_on, engineio_logger=False, reconnection_delay  = 5, reconnection = True) 
    
    
        def set_connection_handler(self, handler):
            self._sio.on('connect',handler = handler)
            
        def set_disconnect_handler(self, handler):
            self._sio.on('disconnect', handler = handler)
    
        def set_hia_events_handler(self, handler):
            self._sio.on('hia_events', handler = handler)
    
    
        def connect(self):
            headers = {
                "jwt_token": self._access_token,
            }
         
            print("CustomerRoom_Api - Connecting to CustomerRoom",)
            self._sio.connect(url = self._socket_url, transports ='websocket', headers=headers, socketio_path= '/target_is_customer_room' ) 
    
        def disconnect(self):
           self._sio.disconnect()
        



# -*- coding: utf-8 -*-
"""
Created on Fri Sep 10 12:37:54 2021

@author: Eli Shamis
"""


from neurospeed.utils.http_service import HttpService

class Auth_AS_User_Handler:

    def __init__(self, user_config):
        self._contex = "User_Auth_Handler - "
        self._user_config = user_config
        self._user_username = self._user_config["username"]
        self._access_token = None
        self._login_status = None
        
        self._auth_api_instance = self.User_Auth_Api(self) # 
    

    
    # login as HIA user
    def login(self):
        try:
            self._login_status = self._auth_api_instance.user_login()
            if (self._login_status == True):
                self._access_token = self._auth_api_instance.get_access_token()
                print('{} successful user login as {}'.format(self._contex, self._user_username) )
            else:
               raise ValueError()
        except:
            print('{} Unable to login as {}'.format(self._contex, self._user_username))
            self._login_status = False
            
        finally:
            return self._login_status
        
    def get_access_token(self):
        return self._access_token
    
    def get_config(self):
        return self._user_config
    
    def get_hia_id(self):
        return self._user_config["HIA_ID"]
    
    def get_username(self):
        return self._user_config["username"]
    
    def is_logged_in(self):
        return self._login_status
    
    def is_verbose_log(self):
        return self._user_config["Verbose_socket_log"] == "True"

    class User_Auth_Api:
    
        def __init__(self, auth_handler_instance):
            self._contex = "User_Auth_Api -"
            
            self._user_config = auth_handler_instance.get_config()
            self._customer_username  = self._user_config["customer_username"]
            self._user_username  = self._user_config["username"]
            self._user_password  = self._user_config["user_password"]

            self._http_service = HttpService()
             
             
        def user_login(self):
            endpoint = "/users/login"
        
            print("{} Executing Login as User {}".format(self._contex, self._user_username))

            login_payload = {
                "customer_username": self._customer_username, 
                "username": self._user_username, 
                "password": self._user_password,
            }
            login_status = False
            try:
                response_payload =  self._http_service.POST_request(endpoint, login_payload)
                
                token = response_payload["token"]
                self._access_token = token["accessToken"]
                # print("User access_token: ", self._access_token)
                login_status = True
            
            except:
                   raise ValueError() 
            
            return login_status
            
        
        def get_access_token(self):
            return self._access_token
        




from neurospeed.utils.helper_service import UtilService as utils
from pathlib import Path
import os

class ApiConfig:

    def __init__(self):
        self._contex = "ApiConfig - "
        self._config_path =  os.path.join(str(Path(__file__).parent.parent) ,"config","api_config.json")
        self._api_config = utils.load_config_file(self._config_path)
        self._api_http_url =  self._api_config["api_address_prod"]  if self._api_config["is_prod"] == "True"  else  self._api_config["api_address_local"] 
        self._pipeline_url =  self._api_config["pipeline_address_prod"]  if self._api_config["is_prod"] == "True"  else  self._api_config["pipeline_address_local"] 
        
       # print("{} NeuroSpeed API HTTP URL: {} ".format(self._contex, self._api_http_url))
       # print("{} NeuroSpeed API SOCKET URL: {} ".format(self._contex, self._pipeline_url))
        
    def get_http_url(self):
        return self._api_http_url
    
    def get_socket_url(self):
        return self._pipeline_url
        


# -*- coding: utf-8 -*-
"""
Created on Tue Sep 14 13:42:26 2021

@author: Eli Shamis
"""

import time
from pathlib import Path
import os
from neurospeed.auth.auth_as_customer_handler import Auth_AS_Customer_handler
from neurospeed.api_http_handlers.recorder_http_handler import UserRoom_Recorder_Handler
from neurospeed.utils.helper_service import UtilService


global customer_auth
global recorder_handler
global username


def run_recorder_flow():
    global recorder_handler
    global username
    # create recorder. recorder created with status "resource_pending". once available recording resources found and assgined
    # recorder status will change to "pending", then once recorder picked by assigned recorder resources and start to record status will change to "recording"

    recorder = recorder_handler.create_recorder(username)
    recorder_id = str(recorder["id"])
    print('created recorder with id', recorder_id)
    
    recorder = recorder_handler.get_recorder(recorder_id)     
    print("recorder status: " + recorder["status"])
    
    recorder_recording = False
    max_query_attemps = 10
    query_attemps = 1
    while (recorder_recording != True & (query_attemps < max_query_attemps) ):
        recorder = recorder_handler.get_recorder(recorder_id)   
        print("recorder status: " + recorder["status"])
        if (recorder["status"] == "recording"):
            print('recorder started recording')
            recorder_recording = True
        else:
            print('recorder not recording yet')
            query_attemps = query_attemps + 1
            time.sleep(3)
             
    if (recorder_recording != True):
        print('recorder not yet recording after ' + str(query_attemps)  + ' query attempts')
        recorder_handler.delete_recorder(recorder_id)
        return
    
    
    print('~~~~~~~~~~~~~~~~~~')
    time.sleep(5)

    #once recorder starts recording, you can change it's status to paused or stopped. you can resume paused recorder but not stopped recorder
    # pause recorder:
    res = recorder_handler.update_recorder(recorder_id, "paused")
    recorder = recorder_handler.get_recorder(recorder_id)     
    print("recorder status: " + recorder["status"])
    time.sleep(5)
    
    # reactivate paused recorder
    recorder_handler.update_recorder(recorder_id, "pending") 
    recorder = recorder_handler.get_recorder(recorder_id)     
    print("recorder status: " + recorder["status"])
    
    time.sleep(5)
    
    #recorder_handler.delete_recorder(recorder_id) # uncomment this to delete recorder
    recorders = recorder_handler.list_recorders(username)
    print(recorders)
    
    return recorder_id

    
    
def main():
    global customer_auth
    global recorder_handler
    global exporter_handler
    global username

    # load Customer configuration
    customer_config_path = os.path.join(str(Path().absolute()) ,"config","customer_config.json")
    customer_config = UtilService.load_config_file(customer_config_path)
    
    # Authenticate as customer
    customer_auth = Auth_AS_Customer_handler(customer_config)
    customer_auth.login()
    
    
    config_user1 = UtilService.load_config_file('config/hia_config1.json')
    username = config_user1["username"] # or set here another custom username

    recorder_handler = UserRoom_Recorder_Handler(customer_auth)
    recorder_id = run_recorder_flow()
    print('RECORDER ID: {}'.format(recorder_id))



if __name__ == '__main__':    
    main()  


# https://python-socketio.readthedocs.io/en/latest/client.html
# https://flask-socketio.readthedocs.io/en/latest/
# https://python-socketio.readthedocs.io/en/latest/intro.html


import queue
import socketio
import json 
import time
import threading
from neurospeed.utils.api_config_service import ApiConfig


class HIA_Client:
    
    _HIA_version = "HIA version 2.0.0" 
    _packet_endpoint = 'pipeline'
    #_streams_dict = {}  # BAD IDEA - this variable would be shared across all class instances, causing data override


    def __init__(self, user_auth_handler, sensor_info):
        self._device_type = "user_data"
        self._hia_id = user_auth_handler.get_hia_id()
        self._username = user_auth_handler.get_username()
        self._contex = "User: {} HIA: {} - ".format(self._username, self._hia_id)
        self._api_config = ApiConfig()
        self._auth_handler = user_auth_handler
        
        # init streams qeueus for each sensor
        self._streams_dict = {} # GOOD IDEA - instance level variable, not shared at class level
        for key, value in sensor_info.items():
            stream_id = key
            sensor_payload =  value
            self._streams_dict[stream_id] = queue.Queue(-1)
            
        self._sensor_info = sensor_info
        self._access_token = self._auth_handler.get_access_token()
        self._user_config = self._auth_handler.get_config()
        self._exit_flag_2 = threading.Event() 
        self._external_socket_connection_handler = None
        self._external_socket_disconnect_handler = None
        self._connection_status = False
        
        logger_on = True
        if self._auth_handler.is_verbose_log() == False:
                logger_on = False
                
        self._sio = socketio.Client(logger=logger_on, engineio_logger=False, reconnection_delay  = 5, reconnection = True) 

        
    # Socket data sender - Start 
    def send_data(self, data, stream_id):  
        out_data = {"timestamp": time.time()} 
        out_data.update(data)

        # insert data into stream queue which belongs to received stream_id
        self._streams_dict[stream_id].put(json.dumps(out_data), timeout = 1)     
           
        
    def send_user_data_to_websocket(self, stream_id):  
        packet_ready = False
        sender_active = True
        while sender_active:
            time.sleep(0.1)
            if self._connection_status is False:
                message = "{} Socket disconnected, stopping user_data upstream for stream_id: {}".format(self._contex, stream_id)
                print(message)
                sender_active = False
                return
            TX_buffer = {"sample": []}
            stream_q_user_data = self._streams_dict[stream_id] # get relevant stream queue
            while (stream_q_user_data.empty() == False):  
                # pull sample from queue, and add to TX buffer:
                sample = json.loads(stream_q_user_data.get(timeout = 1))
                y = TX_buffer['sample']
                y.append(sample)
                packet_ready = True                    
            if (packet_ready):       
               # print("{} user_data packet ready for stream_id: {} ".format(self._contex, stream_id))
                packet_payload = {"stream_id": stream_id, "device_type": self._device_type, "hia_id": self._hia_id, "data": json.dumps(TX_buffer)}
                self._sio.emit(self._packet_endpoint, packet_payload)      
                packet_ready = False
     # Socket data sender - End      
    
    
    # Socket handlers - Start
    def attach_stream_state_handler(self, handler_function):
        self._sio.on('ssr_stream_state',handler = handler_function)
        
    def attach_downlink_handler(self, handler_function):
        self._sio.on('ssr_downlink',handler = handler_function)  
        
    def attach_socket_connection_handler(self, handler_function):
         self._sio.on('connect',handler = handler_function)
    
    def attach_socket_disconnect_handler(self, handler_function):
        self._sio.on('disconnect',handler = handler_function)
    
    def ping_handler(self):  # required DO NOT REMOVE
       self._sio.emit('pong')
       
       
    # internal stream state handler
    def stream_state_handler(self, data):
        print('{} SSR stream_state event: {}'.format(self._contex, data))
        # The incoming packet is a dictionary of shape {"stream_id_x": "stream_state", .., "stream_id_y": "stream_state"}
        # where stream_state is either "disabled" or "enabled" and stream_id is identifier of specific sensor
        str_id = list(data)[0] 
        if str_id in list(self._sensor_info):
            self._sensor_info[str_id]["stream_state"] = data[str_id]
        else:
            print("{} Stream state change error, invalid stream_id: {}".format(self._contex, str_id))
    
    # internal downlink handler
    def downlink_handler(self, payload):
        print('{} SSR downlink event: {}'.format(self._contex, payload))

        
    # internal connection error handler        
    def connection_error_handler(self, data):
        print('{} Connection error, message: {}'.format(self._contex, data) )
        self._connection_status = False
        
    # internal socket disconnect handler
    def socket_disconnect_handler(self):
        self._connection_status = False
        
        # propogate disconnect event to external handler if exist.. 
        if self._external_disconnection_handler != None:
            self._external_disconnection_handler(self) # send current instance contex
        
        
     # internal socket connection handler    
    def socket_connection_handler(self):
        print("{} Connected to NeuroSpeed Pipeline.".format(self._contex))
        self._connection_status = True

        # for each stream inside stream_dict, activate thread for sending data
        for key, value in self._streams_dict.items():
            stream_id = key
            print("{} starting user_data upstream thread for stream_id: {} ".format(self._contex, stream_id))
            user_data_outbound_websocket_TX = threading.Thread(target = self.send_user_data_to_websocket, args=[key])
            time.sleep(0.1)
            user_data_outbound_websocket_TX.start()
        
        # propogate connection event to external handler if exist.. 
        if self._socket_connection_external_handler != None:
            self._socket_connection_external_handler(self) # send current instance contex

   # Socket handlers - End
   
   
   #  Getters, Setters - Start
    def set_socket_connection_external_handler(self, handler):
        self._socket_connection_external_handler = handler
    
    def set_socket_disconnect_external_handler(self, handler):
        self._external_disconnection_handler = handler
        
    def is_connected(self):
        return self._connection_status == True
        
    def update_sensor_info(self, stream_id, payload):
        if (self.is_connected()):
            raise ValueError('Updating sensor info after connection is no allowed. ')
        self._sensor_info[stream_id].update(payload)    
        
    def is_stream_enabled(self, stream_id): 
        return self._sensor_info[stream_id]["stream_state"] == "enabled"
    
    def get_sensor_info(self):
        return self._sensor_info
    
    def get_username(self):
        return self._auth_handler.get_config()["username"]
    
    def get_hia_id(self):
        return self._hia_id
    
    # Self Getters Setters - End
    
    
    def connect(self):
        print(self._HIA_version)
    
        # internal HIA handlers
        self._sio.on('connect_error', handler=self.connection_error_handler)
        self._sio.on('ping', handler=self.ping_handler) #required to identify HIA health by the server ! do not remove.
        self.attach_stream_state_handler(self.stream_state_handler)
        self.attach_downlink_handler(self.downlink_handler)
        self.attach_socket_connection_handler(self.socket_connection_handler)
        self.attach_socket_disconnect_handler(self.socket_disconnect_handler)
        
        headers = {
            "jwt_token": self._access_token,  # required
            "hia_id": self._hia_id,  # required, make sure there are no 2 HIAs with the same id for specific user !
            "sensors_info": json.dumps(self._sensor_info),  # required, at least 1 sensor, otherwise connection will fail
        }
        pipeline_url =  self._api_config.get_socket_url()
     
        print("Connecting to NeuroSpeed pipeline as HIA: {} User: {} Sensors Stream ids: {}"
              .format( self._hia_id, self._username, list(self._sensor_info.keys()) ))
  
        self._sio.connect(url = pipeline_url, transports ='websocket', headers=headers, socketio_path= '/hia' ) 
    

    def disconnect(self):
        self._sio.disconnect()


# -*- coding: utf-8 -*-
"""
Created on Sat Sep 11 20:12:14 2021

@author: Eli Shamis
"""

from neurospeed.utils.http_service import HttpService
import copy 
import json

class Gateway_HttpHandler:
    
    def __init__(self, customer_auth_handler):
        self._customer_auth  = customer_auth_handler
        self._http_service = HttpService()
        
        self._headers =  {
                "Authorization": "Bearer " + customer_auth_handler.get_access_token()
        }

    # returns list of connected users : 
    # [{id: number; username: string;}, ..., {id: number; username: string;}]
    def get_connected_users(self):
        endpoint = "/gateway/users/connected"
        params = {}
        response = self._http_service.GET_request(endpoint, params, self._headers)
        return response

    def get_connections(self, pager, filters): 
         endpoint = "/gateway/connections"
         filters = json.dumps({"filters": filters, "pager": pager})
         headers_copy = copy.deepcopy(self._headers)
         headers_copy["content-type"] = "application/json"
         response = self._http_service.POST_request(endpoint, filters, headers_copy)
         return response
     
        



# -*- coding: utf-8 -*-
"""
Created on Fri Sep 10 12:32:58 2021

@author: Eli Shamis
"""
from pathlib import Path
import os

from neurospeed.auth.auth_as_user_handler import Auth_AS_User_Handler
from neurospeed.api_socket_handlers.user_room_as_user_handler import UserRoom_AS_User_Handler
from neurospeed.utils.helper_service import UtilService

global user_auth



def EEG_processing_handler1(payload):
    # your implementation
    print('EEG_processing_handler1')
    pass;

def EEG_processing_handler2(payload):
    # your implementation
    print('EEG_processing_handler2')
    pass;
    
def customer_gamepad_msg_handler_function(payload):
    # your implementation
    print('customer_gamepad_msg_handler_function')
    pass

def customer_user_data_msg_handler_function(payload):
    # your implementation
    print('customer_user_data_msg_handler_function')
    pass

generic_handler = { # allows multiple handlers for each device
    'eeg': [ EEG_processing_handler1, EEG_processing_handler2],
    'gamepad': [customer_gamepad_msg_handler_function],
    'user_data': [customer_user_data_msg_handler_function]
}



def user_data_external_handler(payload):
    username = user_auth.get_username()
    
    stream_id = payload["stream_id"]
    device_type = payload["device_type"]
    hia_id = payload["hia_id"]
    sensor_info = payload["sensor_info"]
    print("data from user: {} hia: {} stream_id: {} device_type: {}".format(username, hia_id, stream_id, device_type))
    
    # execute each function handler for relevant device_type
    device_type = device_type.lower()
    handler_functions = generic_handler[device_type]
    for func in handler_functions:
        func(payload)
    
# see README(4) for payload structure  
def user_device_event_external_handler(payload):
    print('event: ', payload)
    pass;



# Hint: you can set "Verbose_socket_log": "True" in user config to enable more logging
def main():
    global user_auth

    user1_config_path = os.path.join(str(Path().absolute()) ,"config","hia_config1.json")
    config_user1 = UtilService.load_config_file(user1_config_path)


    # Authenticate as user
    user_auth = Auth_AS_User_Handler(config_user1)
    user_auth.login()

    userRoom = UserRoom_AS_User_Handler(user_auth)
    userRoom.set_data_external_handler(user_data_external_handler)
    userRoom.set_device_events_external_handler(user_device_event_external_handler)

    userRoom.connect()
    


    

if __name__ == '__main__':    
    main()  


# -*- coding: utf-8 -*-
"""
Created on Fri Sep 11 20:07:52 2021

@author: Eli Shamis
"""


import requests
import json
from neurospeed.utils.api_config_service import ApiConfig

class HttpService:    
    
    def __init__(self):
        api_config = ApiConfig()

        self._api_url =  api_config.get_http_url()
        self._api_url = self._api_url + '/api'
    
    
    
    def GET_request(self, endpoint, params, headers = {}):
        api_url = self._api_url + endpoint
        print('doing HTTP GET to url:', api_url, 'with params:', params)
        
        response = None
        try:
            response = requests.get(api_url, headers=headers, params = params)
            response.raise_for_status()
      
            if (response.ok):
                response_parsed = json.loads(response.content)
                print('GET response from {} '.format(endpoint ))
                return response_parsed
    
            else:
                raise requests.exceptions.HTTPError
        
        except requests.exceptions.HTTPError as e:
            print('Failed to HTTP GET to url:', api_url)
            print (e.response.text)
            
            
    def POST_request(self, endpoint, payload, headers = {}):
        api_url = self._api_url + endpoint
        print('doing HTTP POST to url: {} payload {}'.format( api_url, payload ))

        response = None
        try:
            response = requests.post(api_url, data=payload, headers =headers)
            response.raise_for_status()
    
     
            if (response.ok):
                response_parsed = json.loads(response.content)
                print('POST response from {}  '.format(endpoint ))
                return response_parsed
    
            else:
                raise requests.exceptions.HTTPError
        
        except requests.exceptions.HTTPError as e:
            print('Failed to HTTP POST to url:', api_url)
            print (e.response.text)      
            
            
    def PUT_request(self, endpoint, payload, params, headers = {}):
        api_url = self._api_url + endpoint
        print('doing HTTP PUT to url:', api_url, 'with params:', params, 'and payload:', payload, '')
        
        response = None
        try:
            response = requests.put(api_url, data=payload, headers =headers, params=params)
            response.raise_for_status()
            if (response.ok):
                response_parsed = json.loads(response.content)
                print('PUT response from {} '.format(endpoint ))
                return response_parsed
    
            else:
                raise requests.exceptions.HTTPError
        
        except requests.exceptions.HTTPError as e:
            print('Failed to HTTP PUT to url:', api_url)
            print (e.response.text)      
            

    def DELETE_request(self, endpoint, params, headers = {}):
        api_url = self._api_url + endpoint
        print('doing HTTP DELETE to url:', api_url, 'with params:', params)
        
        response = None
        try:
            response = requests.delete(api_url, headers =headers, params=params)
            response.raise_for_status()

            if (response.ok):
                response_parsed = json.loads(response.content)
                print('DELETE response from {} '.format(endpoint ))
                return response_parsed
    
            else:
                raise requests.exceptions.HTTPError
        
        except requests.exceptions.HTTPError as e:
            print('Failed to HTTP DELETE to url:', api_url)
            print (e.response.text)       
  
   


# -*- coding: utf-8 -*-
"""
Created on Fri Sep 11 14:34:45 2021

@author: Eli Shamis
"""


import socketio 
from neurospeed.utils.api_config_service import ApiConfig

class UserRoom_AS_User_Handler:    
    
    def __init__(self, user_auth_handler):
        self._user_auth = user_auth_handler
        self._user_access_token = self._user_auth.get_access_token()
        self._username = self._user_auth.get_username()
        self._data_external_handler = None
        self._event_external_handler = None
        
        # Create UserRoom_Api api instance 
        self._userRoom_api = self.UserRoom_Api(self._user_auth)


    def userRoom_socket_connection_handler(self):
         print("User {} connected to UserRoom".format( self._username ))
    
    def userRoom_socket_disconnect_handler(self):
        print("User {} disconnected from UserRoom".format(self._username ))
    
       
    
    # internal data handler for HIA output data from Neurospeed pipeline    
    def userRoom_data_handler(self, payload):
        stream_id = payload["stream_id"]
        device_type = payload["device_type"]
        hia_id = payload["hia_id"]
        sensor_info = payload["sensor_info"]

        # propogate data to main program (or to any other source, depends on the external_handler callback)
        if (self._data_external_handler != None):
            self._data_external_handler(payload)
            
            
    # receive live events like hia connect\disconnect for this specific user    
    def userRoom_events_handler(self, payload): 
        
         # propogate event to main program (or to any other source, depends on the external_handler callback)
        if (self._events_external_handler != None):
            self._events_external_handler(payload)

        
    def set_data_external_handler(self, handler):
       self._data_external_handler = handler
   
    def set_device_events_external_handler(self, handler):
       self._events_external_handler = handler   



    def connect(self):
        # attach relevant handlers for socket.io events
        self._userRoom_api.set_connection_handler(self.userRoom_socket_connection_handler)
        self._userRoom_api.set_disconnect_handler(self.userRoom_socket_disconnect_handler)
        self._userRoom_api.set_data_handler(self.userRoom_data_handler)
        self._userRoom_api.set_events_handler(self.userRoom_events_handler)
        
        # connect 
        self._userRoom_api.connect()
        
    def disconnect(self):
        self._user_target_room_api.disconnect()
            
    def get_username(self):
        return self._username


    class UserRoom_Api:    
        
        def __init__(self, config):
            api_config = ApiConfig()
            self._socket_url = api_config.get_socket_url()
            
            self._config = config
            self._user_access_token = self._config.get_access_token()
            self._username = self._config.get_username()
            
            logger_on = True
            if self._config.is_verbose_log() == False:
                logger_on = False
                
            self._sio = socketio.Client(logger=logger_on, engineio_logger=False,  reconnection_delay  = 5, reconnection = True) 


    
        def set_connection_handler(self, handler):
            self._sio.on('connect',handler = handler)
            
        def set_disconnect_handler(self, handler):
            self._sio.on('disconnect', handler = handler)
            
        def set_data_handler(self, handler):
            self._sio.on('data', handler = handler)
         
        def set_events_handler(self, handler):
            self._sio.on('events', handler = handler)
            
    
        def connect(self):
            headers = {
                "jwt_token": self._user_access_token,  
            }
         
            print("UserRoom_Api - Connecting to {} UserRoom as USER ".format(self._username))
            self._sio.connect(url = self._socket_url, transports ='websocket', headers=headers, socketio_path= '/target_is_user_room_as_user' ) 
    
        def disconnect(self):
           self._sio.disconnect()


# -*- coding: utf-8 -*-
"""
Created on Fri Sep 10 12:32:58 2021

@author: Eli Shamis
"""

import time
from pathlib import Path
import os

from neurospeed.auth.auth_as_customer_handler import Auth_AS_Customer_handler
from neurospeed.api_http_handlers.gateway_http_handler import Gateway_HttpHandler 
from neurospeed.api_http_handlers.ssr_http_handler import SSR_HttpHandler
from neurospeed.utils.helper_service import UtilService

from neurospeed.api_socket_handlers.customer_room_handler import CustomerRoom_Handler
from neurospeed.api_socket_handlers.user_room_as_customer_handler import UserRoom_AS_Customer_Handler
from neurospeed.api_socket_handlers.downlink_room_as_customer_handler import DownlinkRoom_AS_Customer_Handler

import hia_user_data_sender_example_main as Multiple_Users_HIA_Simulation


global customer_auth
userRooms_dict = {}


def EEG_processing_handler1(payload):
    # your implementation
    print('EEG_processing_handler1')
    pass;
    
def EEG_processing_handler2(payload):
    # your implementation
    print('EEG_processing_handler2')
    pass;
    
def customer_gamepad_msg_handler_function(payload):
    # your implementation
    print('customer_gamepad_msg_handler_function')
    pass

def customer_user_data_msg_handler_function(payload):
    # your implementation
    print('customer_user_data_msg_handler_function')
    pass

# you can add multiple functions into handler array in order to execute multiple callbacks as per your needs
generic_handler = { 
    'eeg': [ EEG_processing_handler1, EEG_processing_handler2 ],
    'gamepad': [customer_gamepad_msg_handler_function],
    'user_data': [customer_user_data_msg_handler_function]
}


def global_users_data_handler(userRoom_instance, payload):
    payload["username"] = userRoom_instance.get_username()
    
    username = payload["username"]
    stream_id = payload["stream_id"]
    device_type = payload["device_type"]
    hia_id = payload["hia_id"]
    sensor_info = payload["sensor_info"]
    
    print("data from user: {} hia: {} stream_id: {} device_type: {}".format(username, hia_id, stream_id, device_type))
    
    # execute each function handler for relevant device_type
    device_type = device_type.lower()
    handler_functions = generic_handler[device_type] 
    for func in handler_functions: 
        func(payload)
    
    
# listen on connected hia events, and connect to userRoom in order to monitor user data
def external_hia_device_connection_handler(event):
    global customer_auth
    global userRooms_dict

    username = event["username"]
    hia_id = event["hia_id"]
    sensors = event["sensors"]
    stream_ids = list(sensors.keys())
    print('external_hia_device_connection_handler event from User: {}, HIA: {} Stream_ids: {}'.format(username ,hia_id, stream_ids ))
    
    if username in userRooms_dict:
       # might be that user already connected with some other HIA device, 
       # so no need to create additional targetRoom for this user
       return
   
    userRoom = UserRoom_AS_Customer_Handler(customer_auth, username)
    # attach external generic handler, in order to propagate user data from "UserRoom_As_Customer_Handler" instance
    # this way every created user_room intance will propagate data to single function where you can apply your custom logic 
    userRoom.set_data_external_handler(global_users_data_handler) 
    userRoom.connect()
    # add created user room to a dict in order to manage and maintain  userRooms
    userRooms_dict[username] = userRoom
    
    
    # Example connect to user machine with SSR_Socket downlink. SSRsocket allows downlink between customer and user on any machine. 
    downlinkRoom = DownlinkRoom_AS_Customer_Handler(customer_auth, username)
    downlinkRoom.connect()
    time.sleep(3)
    if downlinkRoom.is_connected():
        message = "hello user {}".format(username) 
        downlinkRoom.route_message(message)
    
    
    # SSRhttp allows customer to send events to HIA clients (and only to HIA. Device must be connected as HIA)
    # Example of disable\enable stream via SSR HTTP module
    ssr_handler = SSR_HttpHandler(customer_auth)
    
    stream_ids = sensors.keys()
    time.sleep(3)
    # disable streams example
    for stream_id in stream_ids:
        ssr_handler.change_stream_state(username, hia_id, stream_id, "disabled")
        time.sleep(0.2)
    
    # Example SSR downlink that routes message to HIA as is.
    ssr_handler.send_downlink(username, hia_id, "message from SSR HTTP")
    time.sleep(10)    
    # enable streams 
    for stream_id in stream_ids:
        ssr_handler.change_stream_state(username, hia_id, stream_id, "enabled")
        time.sleep(0.2)
    
    
        

def external_hia_device_disconnect_handler(event):
    global customer_auth
    global userRooms_dict

    username = event["username"]
    hia_id = event["hia_id"]
    sensors = event["sensors"]
    stream_ids = list(sensors.keys())
    print('external_user_device_disconnect_handler event from User: {}, HIA: {} Stream_ids: {}'.format(username ,hia_id, stream_ids ))
    
    if username not in userRooms_dict:
        return

    # see readme (1)
    gateway_api = Gateway_HttpHandler(customer_auth)
    connected_users = gateway_api.get_connected_users()
    print("Connected_users: {}".format(connected_users))
    
    user_connected = False
    for connected_user in connected_users:
        connected_username = connected_user["username"]
        if connected_username == username:
            user_connected = True
            
    # if user doesnt exist in the list means he not connected and you can disconnect from his UserRoom
    if user_connected == False:
        userRoom = userRooms_dict[username]
        userRoom.disconnect()
        del userRooms_dict[username]
        

# Hint: you can set "Verbose_socket_log": "True" in customer config to enable more logging
def main():
    global customer_auth
    
    # load customer config file
    customer_config_path = os.path.join(str(Path().absolute()) ,"config","customer_config.json")
    customer_config = UtilService.load_config_file(customer_config_path)
    
    # Authenticate as customer
    customer_auth = Auth_AS_Customer_handler(customer_config)
    customer_auth.login()

    # connect to 'CustomerRoom_Handler', and listen on events across all users
    customerRoom = CustomerRoom_Handler(customer_auth)
    customerRoom.set_hia_connection_external_handler(external_hia_device_connection_handler)
    customerRoom.set_hia_disconnect_external_handler(external_hia_device_disconnect_handler)
    customerRoom.connect()
    
    # see readme (2)
    gateway_api = Gateway_HttpHandler(customer_auth)
    connected_users = gateway_api.get_connected_users()
    print('connected users list from HTTP API:', connected_users)


    # Multiple_Users_HIA_Simulation: 2 USERS, 1 HIA PER USER, 2 SENSORS PER HIA
    # see readme (3)
    # uncomment this in order to run hia example in current console. 
    # or run hia_user_data_sender_example_main.py in another console.
    #Multiple_Users_HIA_Simulation.run_HIAs() 

    

if __name__ == '__main__':    
    main()  


from setuptools import setup, find_packages

setup(
   name='neurospeed',
   version='2.0.6',
   author='Eli Shamis',
   contact_email='oleg@neurobrave.com',
   packages=find_packages(),
   scripts=[],
   package_data={'neurospeed': ['config/*']},
   url='https://bitbucket.org/neurobrave/neurospeed_python_api',
   license='LICENSE.txt',
   description='NeuroSpeed Python API',
   install_requires=[
       "python-socketio[client]==5.3.0"
   ],
)




# -*- coding: utf-8 -*-
"""
Created on Fri Sep 10 12:37:54 2021

@author: Eli Shamis
"""

from neurospeed.utils.http_service import HttpService

class Auth_AS_Customer_handler:

    def __init__(self, customer_config):
        self._contex = "Auth_As_Customer_Handler - "
        self._customer_config = customer_config
        self._access_token = None
        self._customer_email = self._customer_config["email"]
        self._auth_api_instance = self.Customer_Auth_Api(self)

        

    # login as customer
    def login(self):
        try:
            self._login_status = self._auth_api_instance.customer_login()
            if self._login_status == True:
                self._access_token = self._auth_api_instance.get_access_token()
                print('{} Successful login as customer {}'.format(self._contex, self._customer_email) )
            else:
               raise ValueError()
        except:
               print('{} Unable to login as [{}]'.format(self._contex, self._customer_email) )
               self._login_status = False
            
        finally:
            return self._login_status

        
    def get_access_token(self):
        return self._access_token
    
    def get_config(self):
        return self._customer_config

    def get_customer_email(self):
        return self._customer_email
    
    def is_logged_in(self):
        return self._login_status
    
    def get_hia_id(self):
        return self._customer_config["HIA_ID"]

    def is_verbose_log(self):
        return self._customer_config["Verbose_socket_log"] == "True"
 
    
    class Customer_Auth_Api:
        
        def __init__(self, auth_handler_instance):
            self._contex = "Customer_Auth_Api - "
            
            self._customer_config = auth_handler_instance.get_config()
            self._customer_email = self._customer_config["email"]
            self._customer_password  = self._customer_config["password"]
 
            self._http_service = HttpService()
            
        # login api flow
        def customer_login(self):
            endpoint = "/auth/login"
            
            print("{} Executing login flow as customer [{}]".format(self._contex, self._customer_email))
            
            login_payload = {
                "email": self._customer_email, 
                "password": self._customer_password,
            }
            login_status = False
            try:
                response_payload =  self._http_service.POST_request(endpoint, login_payload)
                
                token = response_payload["token"]
                self._access_token = token["accessToken"]
                # print("Customer access_token: ", self._access_token)
                login_status = True
            
            except:
                   raise ValueError() 

      
            return login_status
    
            
        def get_access_token(self):
            return self._access_token
